;; RedMonk/theme.jl
;; Copyright (C) 2000 Unai Uribarri <unaiur@telecable.es>

;; This file is part of sawmill.

;; sawmill is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; sawmill is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with sawmill; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(require 'gradient)

(let*
    ( 
     ;; Icon masks (9x9)
     (close-mask [ 
		 [ () () () () () () () () () ]
		 [ () t  t  () () () () t  t  ]
		 [ () () t  t  () () t  t  () ]
		 [ () () () t  t  t  t  () () ]
		 [ () () () () t  t  () () () ]
		 [ () () () t  t  t  t  () () ]
		 [ () () t  t  () () t  t  () ]
		 [ () t  t  () () () () t  t  ]
		 [ () () () () () () () () () ]])
     
     (max-mask   [
		 [ t  t  t  t  t  t  t  t  t  ]
		 [ t  t  t  t  t  t  t  t  t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  t  t  t  t  t  t  t  t  ]])
     
     (rest-mask  [
		 [ () () t  t  t  t  t  t  () ]
		 [ () () t  t  t  t  t  t  () ]
		 [ () () t  () () () () t  () ]
		 [ t  t  t  t  t  t  () t  () ]
		 [ t  t  t  t  t  t  () t  () ]
		 [ t  () () () () t  t  t  () ]
		 [ t  () () () () t  () () () ]
		 [ t  () () () () t  () () () ]
		 [ t  t  t  t  t  t  () () () ]])
     
     (min-mask   [
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () t  t  t  t  t  t  () () ]
		 [ () t  t  t  t  t  t  () () ]
		 [ () () () () () () () () () ]])
     
     ;; Color definitions ( black white darkgray lightgray title )
     (Windows-95 
      (let ((rgb get-color-rgb))
	(vector (rgb #x0000 #x0000 #x0000) (rgb #xffff #xffff #xffff) 
		(rgb #x8000 #x8000 #x8000) (rgb #xC000 #xC000 #xC000) 
		(rgb #x8000 #x8000 #x8000) (rgb #x8000 #x8000 #x8000)
		(rgb #x0000 #x0000 #x8000) (rgb #x0000 #x0000 #x8000))))
     (Windows-2000 
      (let ((rgb get-color-rgb))
	(vector (rgb #x4000 #x4000 #x4000) (rgb #xffff #xffff #xffff) 
		(rgb #x8000 #x8000 #x8000) (rgb #xD400 #xD000 #xC800) 
		(rgb #x8000 #x8000 #x8000) (rgb #xC000 #xC000 #xC000)
		(rgb #x0A00 #x2400 #x6A00) (rgb #xA600 #xCA00 #xF000))))

     (Bloodlust 
      (let ((rgb get-color-rgb))
	(vector (rgb #x7800 #x0000 #x0000) (rgb #xFF00 #xCF00 #xCF00) 
		(rgb #xB800 #x0000 #x0000) (rgb #xEF00 #x7300 #x7300) 
		(rgb #x4200 #x7000 #x0300) (rgb #x9300 #x4E00 #x0700)
		(rgb #xA400 #x3D00 #x0A00) (rgb #x4800 #x4600 #x0000))))

     (GreenLand 
      (let ((rgb get-color-rgb))
	(vector (rgb #x4000 #x4000 #x4000) (rgb #xffff #xffff #xffff) 
		(rgb #x8000 #x8000 #x8000) (rgb #xD400 #xD000 #xC800) 
		(rgb #x8000 #x8000 #x8000) (rgb #xB400 #xBE00 #xAC00)
		(rgb #x0A00 #x6800 #x0A00) (rgb #xB400 #xE800 #xAC00))))
     
     (Custom
      (let ((rgb get-color-rgb))
	(vector (rgb #x7800 #x0000 #x0000) (rgb #xFF00 #xCF00 #xCF00) 
		(rgb #xB800 #x0000 #x0000) (rgb #xEF00 #x7300 #x7300) 
		(rgb #x4200 #x7000 #x0300) (rgb #x9300 #x4E00 #x0700)
		(rgb #xA400 #x3D00 #x0A00) (rgb #x4800 #x4600 #x0000))))

     (colors GreenLand)

     ;; Color accesors
     (black  (lambda () (aref colors 0)))
     (white  (lambda () (aref colors 1)))
     (dgray  (lambda () (aref colors 2)))
     (lgray  (lambda () (aref colors 3)))
     (dtitle-from (lambda () (aref colors 4)))
     (dtitle-to   (lambda () (aref colors 5)))
     (ltitle-from (lambda () (aref colors 6)))
     (ltitle-to   (lambda () (aref colors 7)))

     ;; Button rendering
     (make-button (lambda (im mask up)
		    (let ((icon-up-gen 
			   (lambda (x y) 
			     (color-rgb-8
			      (cond
			       ((or (= x 15) (= y 13)) (black))
			       ((or (= x 0)  (= y 0))  (white))
			       ((or (= x 14) (= y 12)) (dgray))
			       ((and (>= x 3) (>= y 2) (< x 12) (< y 11))
				(if (aref (aref mask (- y 2)) (- x 3))
				    (black) (lgray)))
			       (t (lgray))))))
			  (icon-dn-gen 
			   (lambda (x y)
			     (color-rgb-8
			      (cond
			       ((or (= x 15) (= y 13)) (white))
			       ((or (= x 0)  (= y 0))  (black))
			       ((or (= x 14) (= y 12)) (lgray))
			       ((or (= x 1)  (= y 1))  (dgray))
			       ((and (>= x 4) (>= y 3) (< x 13) (< y 12))
				(if (aref (aref mask (- y 3)) (- x 4))
				    (black) (lgray)))
			       (t (lgray)))))))
		      (image-fill (if up icon-up-gen icon-dn-gen) im)
		      im)))

					   
     
     ;; images
     ;; gradients
     (dtitle-grad    (make-sized-image (screen-width) 1))
     (ltitle-grad    (make-sized-image (screen-width) 1))
     
     ;; buttons
     (close-btn-up   (make-sized-image 16 14))
     (close-btn-down (make-sized-image 16 14))
     (min-btn-up     (make-sized-image 16 14))
     (min-btn-down   (make-sized-image 16 14))
     (max-btn-up     (make-sized-image 16 14))
     (max-btn-down   (make-sized-image 16 14))
     (rest-btn-up    (make-sized-image 16 14))
     (rest-btn-down  (make-sized-image 16 14))
     
     ;; borders
     (tborder      (make-sized-image 1 4)) ; top border
     (bborder      (make-sized-image 1 4)) ; bottom border
     (lborder      (make-sized-image 4 1)) ; left border
     (rborder      (make-sized-image 4 1)) ; right border
     
     ;; corners
     (tlcorner     (make-sized-image 4 4)) 
     (trcorner     (make-sized-image 4 4)) 
     (blcorner     (make-sized-image 4 4)) 
     (brcorner     (make-sized-image 4 4)) 
     
     ;; Image rendering
     (rebuild-images 
      (lambda ()
	(let ((image-setter 
	       (lambda (img color)
		 (lambda (point) 
		   (image-set img (car point) (cdr point) 
			      (color-rgb-8 color))))))
	  ;; gradients
	  (draw-horizontal-gradient dtitle-grad (dtitle-from) (dtitle-to))
	  (draw-horizontal-gradient ltitle-grad (ltitle-from) (ltitle-to))
	  
	  ;; buttons
	  (make-button close-btn-up   close-mask t)
	  (make-button close-btn-down close-mask nil)
	  (make-button min-btn-up     min-mask t)
	  (make-button min-btn-down   min-mask nil)
	  (make-button max-btn-up     max-mask t)
	  (make-button max-btn-down   max-mask nil)
	  (make-button rest-btn-up    rest-mask t)
	  (make-button rest-btn-down  rest-mask nil)

	  ;; borders
	  (clear-image tborder (lgray))
	  (image-set tborder 0 1 (color-rgb-8 (white)))
	  
	  (clear-image bborder (lgray))
	  (image-set bborder 0 2 (color-rgb-8 (dgray)))
	  (image-set bborder 0 3 (color-rgb-8 (black)))
	  
	  (clear-image lborder (lgray))
	  (image-set lborder 1 0 (color-rgb-8 (white)))

	  (clear-image rborder (lgray))
	  (image-set rborder 2 0 (color-rgb-8 (dgray)))
	  (image-set rborder 3 0 (color-rgb-8 (black)))
		     
	  ;; corners
	  (clear-image tlcorner (lgray))
	  (mapc (image-setter tlcorner (white))
		'((1 . 1)(1 . 2)(1 . 3)(2 . 1)(3 . 1)))

	  (clear-image trcorner (lgray))
	  (mapc (image-setter trcorner (white))
		'((0 . 1)(1 . 1)))
	  (mapc (image-setter trcorner (black))
		'((3 . 0)(3 . 1)(3 . 2)(3 . 3)))
	  (mapc (image-setter trcorner (dgray))
		'((2 . 1)(2 . 2)(2 . 3)))

	  (clear-image blcorner (lgray))
	  (mapc (image-setter blcorner (white))
		'((1 . 0)(1 . 1)))
	  (mapc (image-setter blcorner (black))
		'((0 . 3)(1 . 3)(2 . 3)(3 . 3)))
	  (mapc (image-setter blcorner (dgray))
		'((1 . 2)(2 . 2)(3 . 2)))
	  
	  (clear-image brcorner (black))
	  (mapc (image-setter brcorner (dgray))
		'((0 . 2)(1 . 2)(2 . 2)(2 . 1)(2 . 0)))
	  (mapc (image-setter brcorner (lgray))
		'((0 . 0)(0 . 1)(1 . 0)(1 . 1))))))

     (max-btn   (lambda (win) 
		  (if (window-maximized-p win) 
		      (list rest-btn-up nil nil rest-btn-down)
		    (list max-btn-up nil nil max-btn-down))))

     ;;
     ;; Frame part definitions
     ;;
     (common-parts
      `(;; subtitle-border
	((background . ,lgray)
	 (left-edge  . 0)
	 (right-edge . 0)
	 (top-edge   . -1)
	 (height     . 1))
	;; top-border
	((background  . ,tborder)
	 (top-edge    . -23)
	 (left-edge   . 0)
	 (right-edge  . 0)
	 (height      . 4)
	 (class       . top-border))
	;; top-left-corner
	((background . ,tlcorner)
	 (top-edge   . -23)
	 (left-edge  . -4)
	 (height     . 4)
	 (width      . 4)
	 (class      . top-left-corner))
	;; top-right-corner
	((background . ,trcorner)
	 (top-edge   . -23)
	 (right-edge . -4)
	 (height     . 4)
	 (width      . 4)
	 (class      . top-right-corner))))

     (shaped-parts
      `(;; left-up-border
	((background  . ,lborder)
	 (top-edge    . -19)
	 (left-edge   . -4)
	 (width       . 4)
	 (height      . 19)
	 (class       . left-border))
	;; right-up-border
	((background  . ,rborder)
	 (top-edge    . -19)
	 (right-edge . -4)
	 (width      . 4)
	 (height      . 19)
	 (class      . right-border))
	. ,common-parts))

     (non-shaped-parts
      `(;; bottom-border
	((background  . ,bborder)
	 (bottom-edge . -4)
	 (left-edge   . 0)
	 (right-edge  . 0)
	 (height      . 4)
	 (class       . bottom-border))
	;; left-border
	((background  . ,lborder)
	 (top-edge    . -19)
	 (bottom-edge . 0)
	 (left-edge   . -4)
	 (width       . 4)
	 (class       . left-border))
	;; right-border
	((background  . ,rborder)
	 (top-edge    . -19)
	 (bottom-edge . 0)
	 (right-edge . -4)
	 (width      . 4)
	 (class      . right-border))
	;; buttom-left-corner
	((background  . ,blcorner)
	 (bottom-edge . -4)
	 (left-edge   . -4)
	 (height      . 4)
	 (width       . 4)
	 (class       . bottom-left-corner))
	;; buttom-left-corner
	((background  . ,brcorner)
	 (bottom-edge . -4)
	 (right-edge  . -4)
	 (height      . 4)
	 (width       . 4)
	 (class       . bottom-right-corner))
	. ,common-parts))

     (transient-parts
      `(;; iconify button
	((foreground . ,(list min-btn-up nil nil min-btn-down))
	 (background . ,(black))
	 (y-justify  . center)
	 (top-edge   . -17)
	 (right-edge . 2)
	 (width      . 16)
	 (height     . 14)
	 (removable  . true)
	 (class      . iconify-button))))

     (non-transient-parts
      `(;;Close
	((foreground . ,(list close-btn-up close-btn-up 
			      close-btn-up close-btn-down))
	 (background . ,(lgray))
	 (x-justify  . center)
	 (y-justify  . center)
	 (top-edge   . -17)
	 (right-edge . 2)
	 (width      . 16)
	 (height     . 14)
	 (removable  . true)
	 (class      . close-button))
	;;Maximize-Restore
	((foreground . ,max-btn)
	 (background . ,(black))
	 (y-justify  . center)
	 (top-edge   . -17)
	 (right-edge . 20)
	 (width      . 16)
	 (height     . 14)
	 (removable  . true)
	 (class      . maximize-button))
	;;Iconify
	((foreground . ,(list min-btn-up nil nil min-btn-down))
	 (background . ,(black))
	 (y-justify  . center)
	 (top-edge   . -17)
	 (right-edge . 36)
	 (width      . 16)
	 (height     . 14)
	 (removable  . true)
	 (class      . iconify-button))))

     (shaped-transient-frame (append transient-parts shaped-parts))
     (shaped-frame    (append non-transient-parts shaped-parts))
     (transient-frame (append transient-parts non-shaped-parts))
     (default-frame   (append non-transient-parts non-shaped-parts))

     ;; Icon handling
     (default-icon (make-image "icon.png"))
     (get-win-icon (lambda (win)
		     (let ((icon (window-icon-image win)))
		       (if icon icon default-icon))))
     
     (rebuild-theme (lambda ()
		      (setq colors (symbol-value RedMonk:ColorMap))
		      (rebuild-images)
		      (reframe-windows-with-style 'RedMonk))))
  
  (rebuild-images)
  
  (add-frame-style 
   'RedMonk 
   (lambda (win type)
     `(
       ;; title part
       ((background . (,dtitle-grad ,ltitle-grad))
	(foreground . (,(lgray) ,(white)))
	(text       . ,window-name)
	(x-justify  . 22)
	(y-justify  . center)
	(left-edge  . 0)
	(right-edge . 0)
	(top-edge   . -19)
	(height     . 18)
	(below-client . t)
	(class      . title))
       ;; icon
       ((background . ,(get-win-icon win))
	(x-justify  . center)
	(y-justify  . center)
	(top-edge   . -18)
	(left-edge  . 2)
	(width      . 16)
	(height     . 16)
	(class      . menu-button))
       . ,(case type
	    ((transient) transient-frame)
	    ((shaped shaded) shaped-frame)
	    ((shaped-transient) shaped-transient-frame)
	    (t default-frame)))))
  

  ;; Hooks
  (call-after-property-changed 
   'WM_NAME (lambda (win) (rebuild-frame win)))
  (call-after-property-changed
   'WM_HINTS (lambda (win) (reframe-window win)))


  (defgroup RedMonk "RedMonk Theme"
    :group appearance)
  
  (defcustom RedMonk:ColorMap 'GreenLand
    "Color map: \\w"
    :type (set Windows-2000 Windows-95 GreenLand Bloodlust Custom)
    :group (appearance RedMonk)
    :user-level novice
    :after-set rebuild-theme)

  ;; Custom color map settings
  (defcustom RedMonk:BlackColor (aref Custom 0)
    "Custom color map: \\w Black substitution color"
    :type color
    :group (appearance RedMonk)
    :user-level expert
    :after-set (lambda (X)
		 (aset Custom 0 RedMonk:BlackColor)
		 (if (eq RedMonk:ColorMap 'Custom)
		     (rebuild-theme))))
  (defcustom RedMonk:WhiteColor (aref Custom 1)
    "Custom color map: \\w White substitution color"
    :type color
    :group (appearance RedMonk)
    :user-level expert
    :after-set (lambda (X)
		 (aset Custom 1 RedMonk:WhiteColor)
		 (if (eq RedMonk:ColorMap 'Custom)
		     (rebuild-theme))))
  (defcustom RedMonk:DarkColor (aref Custom 2)
    "Custom color map: \\w Dark gray substitution color"
    :type color
    :group (appearance RedMonk)
    :user-level expert
    :after-set (lambda (X)
		 (aset Custom 2 RedMonk:DarkColor)
		 (if (eq RedMonk:ColorMap 'Custom)
		     (rebuild-theme))))
  (defcustom RedMonk:LightColor (aref Custom 3)
    "Custom color map: \\w Light gray substitution color"
    :type color
    :group (appearance RedMonk)
    :user-level expert
    :after-set (lambda (X)
		 (aset Custom 3 RedMonk:LightColor)
		 (if (eq RedMonk:ColorMap 'Custom)
		     (rebuild-theme))))
  (defcustom RedMonk:FocusedTitleFromColor (aref Custom 6)
    "Custom color map: \\w Focused title at left"
    :type color
    :group (appearance RedMonk)
    :user-level expert
    :after-set (lambda (X)
		 (aset Custom 6 RedMonk:FocusedTitleFromColor)
		 (if (eq RedMonk:ColorMap 'Custom)
		     (rebuild-theme))))
  (defcustom RedMonk:FocusedTitleToColor (aref Custom 7)
    "Custom color map: \\w Focused title at right"
    :type color
    :group (appearance RedMonk)
    :user-level expert
    :after-set (lambda (X)
		 (aset Custom 7 RedMonk:FocusedTitleToColor)
		 (if (eq RedMonk:ColorMap 'Custom)
		     (rebuild-theme))))
  (defcustom RedMonk:UnfocusedTitleFromColor (aref Custom 4)
    "Custom color map: \\w Unfocused title at left"
    :type color
    :group (appearance RedMonk)
    :user-level expert
    :after-set (lambda (X)
		 (aset Custom 4 RedMonk:UnfocusedTitleFromColor)
		 (if (eq RedMonk:ColorMap 'Custom)
		     (rebuild-theme))))
  (defcustom RedMonk:UnfocusedTitleToColor (aref Custom 5)
    "Custom color map: \\w Unfocused title at right"
    :type color
    :group (appearance RedMonk)
    :user-level expert
    :after-set (lambda (X)
		 (aset Custom 5 RedMonk:UnfocusedTitleToColor)
		 (if (eq RedMonk:ColorMap 'Custom)
		     (rebuild-theme)))))
